@@ -195,6 +195,50 @@ class SkipProtectionWhenUnprotectedController < ActionController::Base
   skip_forgery_protection
 end
 
+class CookieCsrfTokenStorageStrategyController < ActionController::Base
+  include RequestForgeryProtectionActions
+
+  after_action :commit_token, only: :cookie
+
+  protect_from_forgery only: %w(index meta same_origin_js negotiate_same_origin), with: :exception, store: :cookie
+
+  def reset
+    reset_csrf_token(request)
+    head :ok
+  end
+
+  def cookie
+    render inline: "<%= csrf_meta_tags %>"
+  end
+
+  private
+    def commit_token
+      request.commit_csrf_token
+    end
+end
+
+class CustomCsrfTokenStorageStrategyController < ActionController::Base
+  include RequestForgeryProtectionActions
+
+  class CustomStrategy
+    def fetch(request)
+      request.env[:custom_storage]
+    end
+
+    def store(request, csrf_token)
+      request.env[:custom_storage] = csrf_token
+    end
+
+    def reset(request)
+      request.env[:custom_storage] = nil
+    end
+  end
+
+  protect_from_forgery only: %w(index meta same_origin_js negotiate_same_origin),
+    with: :reset_session,
+    store: CustomStrategy.new
+end
+
 # common test methods
 module RequestForgeryProtectionTests
   def setup
@@ -394,7 +438,7 @@ module RequestForgeryProtectionTests
   end
 
   def test_should_allow_post_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
     end
@@ -403,85 +447,60 @@ module RequestForgeryProtectionTests
   def test_should_allow_post_with_strict_encoded_token
     token_length = (ActionController::RequestForgeryProtection::AUTHENTICITY_TOKEN_LENGTH * 4.0 / 3).ceil
     token_including_url_unsafe_chars = "+/".ljust(token_length, "A")
-    session[:_csrf_token] = token_including_url_unsafe_chars
+    initialize_csrf_token(token_including_url_unsafe_chars)
     @controller.stub :form_authenticity_token, token_including_url_unsafe_chars do
       assert_not_blocked { post :index, params: { custom_authenticity_token: token_including_url_unsafe_chars } }
     end
   end
 
-  def test_should_allow_post_with_urlsafe_token_when_migrating
-    ActiveSupport::Deprecation.silence do
-      ActionController::Base.urlsafe_csrf_tokens = false
-    end
-    token_length = (ActionController::RequestForgeryProtection::AUTHENTICITY_TOKEN_LENGTH * 4.0 / 3).ceil
-    token_including_url_safe_chars = "-_".ljust(token_length, "A")
-    session[:_csrf_token] = token_including_url_safe_chars
-    @controller.stub :form_authenticity_token, token_including_url_safe_chars do
-      assert_not_blocked { post :index, params: { custom_authenticity_token: token_including_url_safe_chars } }
-    end
-  ensure
-    ActiveSupport::Deprecation.silence do
-      ActionController::Base.urlsafe_csrf_tokens = true
-    end
-  end
-
-  def test_should_warn_about_deprecation_for_urlsafe_config
-    assert_deprecated do
-      ActionController::Base.urlsafe_csrf_tokens = false
-    end
-    assert_deprecated do
-      ActionController::Base.urlsafe_csrf_tokens = true
-    end
-  end
-
   def test_should_allow_patch_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { patch :index, params: { custom_authenticity_token: @token } }
     end
   end
 
   def test_should_allow_put_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { put :index, params: { custom_authenticity_token: @token } }
     end
   end
 
   def test_should_allow_delete_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { delete :index, params: { custom_authenticity_token: @token } }
     end
   end
 
   def test_should_allow_post_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { post :index }
   end
 
   def test_should_allow_delete_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { delete :index }
   end
 
   def test_should_allow_patch_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { patch :index }
   end
 
   def test_should_allow_put_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { put :index }
   end
 
   def test_should_allow_post_with_origin_checking_and_correct_origin
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         assert_not_blocked do
           @request.set_header "HTTP_ORIGIN", "http://test.host"
@@ -493,7 +512,7 @@ module RequestForgeryProtectionTests
 
   def test_should_allow_post_with_origin_checking_and_no_origin
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         assert_not_blocked do
           post :index, params: { custom_authenticity_token: @token }
@@ -504,7 +523,7 @@ module RequestForgeryProtectionTests
 
   def test_should_raise_for_post_with_null_origin
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         exception = assert_raises(ActionController::InvalidAuthenticityToken) do
           @request.set_header "HTTP_ORIGIN", "null"
@@ -521,7 +540,7 @@ module RequestForgeryProtectionTests
     ActionController::Base.logger = logger
 
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         assert_blocked do
           @request.set_header "HTTP_ORIGIN", "http://bad.host"
@@ -538,6 +557,7 @@ module RequestForgeryProtectionTests
     ActionController::Base.logger = old_logger
   end
 
+
   def test_should_warn_on_missing_csrf_token
     old_logger = ActionController::Base.logger
     logger = ActiveSupport::LogSubscriber::TestHelper::MockLogger.new
@@ -623,7 +643,7 @@ module RequestForgeryProtectionTests
 
   # Allow non-GET requests since GET is all a remote <script> tag can muster.
   def test_should_allow_non_get_js_without_xhr_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     assert_cross_origin_not_blocked { post :same_origin_js, params: { custom_authenticity_token: @token } }
     assert_cross_origin_not_blocked { post :same_origin_js, params: { format: "js", custom_authenticity_token: @token } }
     assert_cross_origin_not_blocked do
@@ -648,12 +668,25 @@ module RequestForgeryProtectionTests
     end
   end
 
+  def test_csrf_token_is_not_saved_if_it_is_nil
+    @controller.commit_csrf_token(@request)
+    assert_nil fetch_csrf_token
+  end
+
   def test_should_not_raise_error_if_token_is_not_a_string
     assert_blocked do
       patch :index, params: { custom_authenticity_token: { foo: "bar" } }
     end
   end
 
+  def initialize_csrf_token(token = @token, session = self.session)
+    session[:_csrf_token] = token
+  end
+
+  def fetch_csrf_token
+    session[:_csrf_token]
+  end
+
   def assert_blocked
     session[:something_like_user_id] = 1
     yield
@@ -662,7 +695,9 @@ module RequestForgeryProtectionTests
   end
 
   def assert_not_blocked(&block)
+    session[:something_like_user_id] = 1
     assert_nothing_raised(&block)
+    assert_equal 1, session[:something_like_user_id]
     assert_response :success
   end
 
@@ -738,7 +773,7 @@ class RequestForgeryProtectionControllerUsingExceptionTest < ActionController::T
 
   def test_raised_exception_message_explains_why_it_occurred
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         exception = assert_raises(ActionController::InvalidAuthenticityToken) do
           @request.set_header "HTTP_ORIGIN", "http://bad.host"
@@ -885,7 +920,7 @@ class PerFormTokensControllerTest < ActionController::TestCase
   def test_per_form_token_is_same_size_as_global_token
     get :index
     expected = ActionController::RequestForgeryProtection::AUTHENTICITY_TOKEN_LENGTH
-    actual = @controller.send(:per_form_csrf_token, session, "/path", "post").size
+    actual = @controller.send(:per_form_csrf_token, nil, "/path", "post").size
     assert_equal expected, actual
   end
 
@@ -1023,7 +1058,7 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     unmasked_token = @controller.send(:unmask_token, Base64.urlsafe_decode64(token))
 
-    assert_not_equal @controller.send(:real_csrf_token, session), unmasked_token
+    assert_not_equal @controller.send(:real_csrf_token), unmasked_token
   end
 
   def test_returns_hmacd_token
@@ -1033,13 +1068,13 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     unmasked_token = @controller.send(:unmask_token, Base64.urlsafe_decode64(token))
 
-    assert_equal @controller.send(:global_csrf_token, session), unmasked_token
+    assert_equal @controller.send(:global_csrf_token), unmasked_token
   end
 
   def test_accepts_old_csrf_token
     get :index
 
-    non_hmac_token = @controller.send(:mask_token, @controller.send(:real_csrf_token, session))
+    non_hmac_token = @controller.send(:mask_token, @controller.send(:real_csrf_token))
 
     # This is required because PATH_INFO isn't reset between requests.
     @request.env["PATH_INFO"] = "/per_form_tokens/post_one"
@@ -1126,7 +1161,7 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     def assert_matches_session_token_on_server(form_token, method = "post")
       actual = @controller.send(:unmask_token, Base64.urlsafe_decode64(form_token))
-      expected = @controller.send(:per_form_csrf_token, session, "/per_form_tokens/post_one", method)
+      expected = @controller.send(:per_form_csrf_token, nil, "/per_form_tokens/post_one", method)
       assert_equal expected, actual
     end
 end
@@ -1162,3 +1197,174 @@ class SkipProtectionWhenUnprotectedControllerTest < ActionController::TestCase
     assert_response :success
   end
 end
+
+class CookieCsrfTokenStorageStrategyControllerTest < ActionController::TestCase
+  include RequestForgeryProtectionTests
+
+  class TestSession < ActionController::TestSession
+    attr_reader :id_was
+
+    def initialize(id_was)
+      super()
+      @id_was = id_was
+    end
+  end
+
+  class NullSessionDummyKeyGenerator
+    def generate_key(secret, length = nil)
+      "03312270731a2ed0d11ed091c2338a06"
+    end
+  end
+
+  def setup
+    @request.env[ActionDispatch::Cookies::GENERATOR_KEY] = NullSessionDummyKeyGenerator.new
+    @request.env[ActionDispatch::Cookies::COOKIES_ROTATIONS] = ActiveSupport::Messages::RotationConfiguration.new
+    super
+  end
+
+  def test_csrf_token_is_stored_in_cookie
+    get :cookie
+    assert_not session.key?(:_csrf_token)
+    assert cookies.key?(:csrf_token)
+  end
+
+  def test_csrf_token_is_stored_in_custom_cookie
+    @controller.csrf_token_storage_strategy =
+      ActionController::RequestForgeryProtection::CookieStore.new(:custom_cookie)
+    get :cookie
+    assert_not cookies.key?(:csrf_token)
+    assert cookies.key?(:custom_cookie)
+  end
+
+  def test_csrf_token_cookie_has_same_site_lax
+    get :cookie
+    assert_set_cookie_attributes("csrf_token", "SameSite=Lax")
+  end
+
+  include CookieAssertions
+
+  def test_csrf_token_cookie_is_http_only
+    get :cookie
+
+    cookies = parse_set_cookies_headers(@response.headers["Set-Cookie"])
+    csrf_token_cookie = cookies["csrf_token"]
+    assert csrf_token_cookie["httponly"]
+  end
+
+  def test_csrf_token_cookie_is_permanent
+    get :cookie
+    assert_match(%r(#{20.years.from_now.utc.year}), @response.headers["Set-Cookie"])
+  end
+
+  def test_reset_csrf_token_deletes_cookie
+    get :cookie
+    get :reset
+    assert_nil cookies[:csrf_token]
+  end
+
+  def test_should_allow_when_session_id_in_cookie_matches_session_id
+    initialize_csrf_token
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_not_allow_when_session_id_in_cookie_does_not_match_session_id
+    initialize_csrf_token(@token, ActionController::TestSession.new)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_allow_when_session_id_in_cookie_and_session_id_are_nil
+    @request.session = ActionController::TestSession.new({}, nil)
+    initialize_csrf_token(@token, nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_not_allow_when_session_id_in_cookie_but_session_id_is_nil
+    initialize_csrf_token
+    @request.session = ActionController::TestSession.new({}, nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_allow_when_session_id_in_cookie_is_nil_and_session_created_before_token_validation
+    initialize_csrf_token(@token, nil)
+    @request.session = TestSession.new(nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_allow_when_session_id_in_cookie_is_nil_and_session_reset_before_token_validation
+    initialize_csrf_token
+    @request.session = TestSession.new(session.id)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_not_allow_when_session_id_in_cookie_but_request_made_with_no_session
+    initialize_csrf_token
+    @request.session = TestSession.new(nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def initialize_csrf_token(token = @token, session = self.session)
+    cookies.encrypted[:csrf_token] = {
+      value: {
+        token: token,
+        session_id: session&.id,
+      }.to_json,
+      httponly: true,
+      same_site: :lax,
+    }
+  end
+
+  def fetch_csrf_token
+    contents = request.cookie_jar.encrypted[:csrf_token]
+    return nil if contents.nil?
+
+    value = JSON.parse(contents)
+    return nil unless value["session_id"]&.fetch("public_id") == request.session.id_was&.public_id
+
+    value["token"]
+  end
+
+  def assert_blocked(&block)
+    assert_raises(ActionController::InvalidAuthenticityToken, &block)
+  end
+
+  def assert_not_blocked(&block)
+    assert_nothing_raised(&block)
+    assert_response :success
+  end
+end
+
+class CustomCsrfTokenStorageStrategyControllerTest < ActionController::TestCase
+  include RequestForgeryProtectionTests
+
+  def test_csrf_token_is_stored_in_custom_location
+    post :index
+    @controller.commit_csrf_token(@request)
+    assert_not session.key?(:_csrf_token)
+    assert_not_nil request.env[:custom_storage]
+  end
+
+  def initialize_csrf_token(token = @token)
+    request.env[:custom_storage] = token
+  end
+end
