@@ -115,11 +115,6 @@ class CookiesTest < ActionController::TestCase
       head :ok
     end
 
-    def set_with_with_escapable_characters
-      cookies["that & guy"] = "foo & bar => baz"
-      head :ok
-    end
-
     def authenticate_for_fourteen_days
       cookies["user_name"] = { "value" => "david", "expires" => Time.utc(2005, 10, 10, 5) }
       head :ok
@@ -493,12 +488,6 @@ class CookiesTest < ActionController::TestCase
     assert_equal({ "user_name" => "Jamie" }, response.cookies)
   end
 
-  def test_setting_with_escapable_characters
-    get :set_with_with_escapable_characters
-    assert_set_cookie_header "that+%26+guy=foo+%26+bar+%3D%3E+baz; path=/; SameSite=Lax"
-    assert_equal({ "that & guy" => "foo & bar => baz" }, @response.cookies)
-  end
-
   def test_setting_cookie_for_fourteen_days
     get :authenticate_for_fourteen_days
     assert_set_cookie_header "user_name=david; path=/; expires=Mon, 10 Oct 2005 05:00:00 GMT; SameSite=Lax"
