@@ -122,7 +122,8 @@ module RenderTestCases
     e = assert_raise ActionView::Template::Error do
       @view.render(template: "with_format", formats: [:json])
     end
-    assert_includes(e.message, "Missing partial /_missing with {:locale=>[:en], :formats=>[:json], :variants=>[], :handlers=>[:raw, :erb, :html, :builder, :ruby]}.")
+    details = { locale: [:en], formats: [:json], variants: [], handlers: [:raw, :erb, :html, :builder, :ruby] }
+    assert_includes(e.message, "Missing partial /_missing with #{details}.")
   end
 
   def test_render_template_with_locale
@@ -151,7 +152,7 @@ module RenderTestCases
     buffer = ActiveSupport::SafeBuffer.new
     buffer << @view.render(template: "plain_text")
     assert_equal true, buffer.html_safe?
-    assert_equal buffer, "<%= hello_world %>\n"
+    assert_equal "<%= hello_world %>\n", buffer
   end
 
   def test_render_ruby_template_with_handlers
@@ -204,9 +205,48 @@ module RenderTestCases
   def test_render_outside_path
     assert File.exist?(File.expand_path("../../test/abstract_unit.rb", __dir__))
     assert_raises ActionView::MissingTemplate do
-      assert_deprecated do
-        @view.render(template: "../\\../test/abstract_unit.rb")
-      end
+      @view.render(template: "../\\../test/abstract_unit.rb")
+    end
+  end
+
+  if RUBY_VERSION >= "3.2" && RUBY_VERSION < "3.4" # https://github.com/rails/rails/issues/52902
+    def test_render_runtime_error
+      ex = assert_raises(ActionView::Template::Error) {
+        @view.render(template: "test/runtime_error")
+      }
+      erb_btl = ex.backtrace_locations.first
+
+      # Get the spot information from ErrorHighlight
+      translating_frame = ActionDispatch::ExceptionWrapper::SourceMapLocation.new(erb_btl, ex.template)
+      translated_spot = translating_frame.spot(ex.cause)
+
+      assert_equal 6, translated_spot[:first_column]
+    end
+
+    def test_render_location_conditional_append
+      ex = assert_raises(ActionView::Template::Error) {
+        @view.render(template: "test/unparseable_runtime_error")
+      }
+      erb_btl = ex.backtrace_locations.first
+
+      # Get the spot information from ErrorHighlight
+      translating_frame = ActionDispatch::ExceptionWrapper::SourceMapLocation.new(erb_btl, ex.template)
+      translated_spot = translating_frame.spot(ex.cause)
+
+      assert_equal 8, translated_spot[:first_column]
+    end
+
+    def test_render_location_conditional_append_2
+      ex = assert_raises(ActionView::Template::Error) {
+        @view.render(template: "test/unparseable_runtime_error_2")
+      }
+      erb_btl = ex.backtrace_locations.first
+
+      # Get the spot information from ErrorHighlight
+      translating_frame = ActionDispatch::ExceptionWrapper::SourceMapLocation.new(erb_btl, ex.template)
+      translated_spot = translating_frame.spot(ex.cause)
+
+      assert_instance_of Integer, translated_spot[:first_column]
     end
   end
 
@@ -290,13 +330,15 @@ module RenderTestCases
     assert_equal "1:    <%= foo(", e.annotated_source_code[0].strip
   end
 
-  def test_render_partial_with_errors
-    e = assert_raises(ActionView::Template::Error) { @view.render(partial: "test/raise") }
-    assert_match %r!method.*doesnt_exist!, e.message
-    assert_equal "", e.sub_template_message
-    assert_equal "1", e.line_number
-    assert_equal "1: <%= doesnt_exist %>", e.annotated_source_code[0].strip
-    assert_equal File.expand_path("#{FIXTURE_LOAD_PATH}/test/_raise.html.erb"), e.file_name
+  if RUBY_VERSION < "3.4" # https://github.com/rails/rails/issues/52902
+    def test_render_partial_with_errors
+      e = assert_raises(ActionView::Template::Error) { @view.render(partial: "test/raise") }
+      assert_match %r!method.*doesnt_exist!, e.message
+      assert_equal "", e.sub_template_message
+      assert_equal "1", e.line_number
+      assert_equal "1: <%= doesnt_exist %>", e.annotated_source_code[0].strip
+      assert_equal File.expand_path("#{FIXTURE_LOAD_PATH}/test/_raise.html.erb"), e.file_name
+    end
   end
 
   def test_render_error_indentation
@@ -309,7 +351,7 @@ module RenderTestCases
   end
 
   def test_render_template_with_errors
-    e = assert_raises(ActionView::Template::Error) { assert_deprecated { @view.render(template: "test/_raise") } }
+    e = assert_raises(ActionView::Template::Error) { @view.render(template: "test/_raise") }
     assert_match %r!method.*doesnt_exist!, e.message
     assert_equal "", e.sub_template_message
     assert_equal "1", e.line_number
@@ -327,7 +369,7 @@ module RenderTestCases
 
   def test_undefined_method_error_references_named_class
     e = assert_raises(ActionView::Template::Error) { @view.render(inline: "<%= undefined %>") }
-    assert_match(/undefined local variable or method `undefined'/, e.message)
+    assert_match(/undefined local variable or method [`']undefined'/, e.message)
   end
 
   def test_render_renderable_object
@@ -512,7 +554,7 @@ module RenderTestCases
   end
 
   def test_optional_second_arg_works_without_deprecation
-    assert_not_deprecated do
+    assert_not_deprecated(ActionView.deprecator) do
       ActionView::Template.register_template_handler :ruby_handler, ->(view, source = nil) { source }
     end
     assert_equal "3", @view.render(inline: "(1 + 2).to_s", type: :ruby_handler)
@@ -563,9 +605,7 @@ module RenderTestCases
     %w(malformed malformed.erb malformed.html.erb malformed.en.html.erb).each do |name|
       assert File.exist?(File.expand_path("#{FIXTURE_LOAD_PATH}/test/malformed/#{name}~")), "Malformed file (#{name}~) which should be ignored does not exists"
       assert_raises(ActionView::MissingTemplate) do
-        ActiveSupport::Deprecation.silence do
-          @view.render(template: "test/malformed/#{name}")
-        end
+        @view.render(template: "test/malformed/#{name}")
       end
     end
   end
@@ -935,6 +975,12 @@ class CachedCollectionViewRenderTest < ActiveSupport::TestCase
     end
   end
 
+  test "collection caching with empty collection and logger with level debug" do
+    ActionView::PartialRenderer.collection_cache.logger = Logger.new(nil, level: :debug)
+
+    assert_nil @view.render(partial: "test/cached_customer", collection: [], cached: true)
+  end
+
   test "collection caching with repeated collection" do
     sets = [
         [1, 2, 3, 4, 5],
