@@ -217,50 +217,6 @@ class TagHelperTest < ActionView::TestCase
     assert_equal '<input pattern="\w+">', tag.input(pattern: /\w+/)
   end
 
-  def test_deprecated_option_escape_attributes
-    raw_content = "<script>evil_js</script>"
-    escaped_content = "&lt;script&gt;evil_js&lt;/script&gt;"
-
-    expected_deprecation_warning = <<~MSG
-      Use of the option :escape_attributes is deprecated. It currently \
-      escapes both names and values of tags and attributes and it is \
-      equivalent to :escape. If any of them are enabled, the escaping \
-      is fully enabled.
-    MSG
-
-    assert_equal "<p>#{escaped_content}</p>",
-                 tag.p(raw_content)
-    assert_equal "<p>#{escaped_content}</p>",
-                 tag.p(raw_content, escape: true)
-    assert_equal "<p>#{raw_content}</p>",
-                 tag.p(raw_content, escape: false)
-
-    assert_deprecated(expected_deprecation_warning) do
-      assert_equal "<p>#{escaped_content}</p>",
-                   tag.p(raw_content, escape_attributes: true)
-    end
-    assert_deprecated(expected_deprecation_warning) do
-      assert_equal "<p>#{raw_content}</p>",
-                   tag.p(raw_content, escape_attributes: false)
-    end
-    assert_deprecated(expected_deprecation_warning) do
-      assert_equal "<p>#{escaped_content}</p>",
-                   tag.p(raw_content, escape_attributes: true, escape: true)
-    end
-    assert_deprecated(expected_deprecation_warning) do
-      assert_equal "<p>#{raw_content}</p>",
-                   tag.p(raw_content, escape_attributes: false, escape: false)
-    end
-    assert_deprecated(expected_deprecation_warning) do
-      assert_equal "<p>#{escaped_content}</p>",
-                   tag.p(raw_content, escape_attributes: true, escape: false)
-    end
-    assert_deprecated(expected_deprecation_warning) do
-      assert_equal "<p>#{escaped_content}</p>",
-                   tag.p(raw_content, escape_attributes: false, escape: true)
-    end
-  end
-
   def test_tag_builder_nested
     assert_equal "<div>content</div>",
                  tag.div { "content" }
@@ -342,6 +298,11 @@ class TagHelperTest < ActionView::TestCase
     assert_equal "<p>\n  <b>Hello</b>\n</p>", view.render("test/builder_tag_nested_in_content_tag")
   end
 
+  def test_content_tag_nested_in_content_tag_with_data_attributes_out_of_erb
+    assert_equal "<div data-controller=\"read-more\" data-read-more-more-text-value=\"Read more\" data-read-more-less-text-value=\"Read less\"\>\n  <p class=\"content-class\" data-test-name=\"content\">Content text</p>\n  <button class=\"expand-button\" data-action=\"read-more#toggle\">Read more</button>\n</div>",
+                  view.render("test/content_tag_nested_in_content_tag_with_data_attributes_out_of_erb")
+  end
+
   def test_content_tag_with_escaped_array_class
     str = content_tag("p", "limelight", class: ["song", "play>"])
     assert_equal "<p class=\"song play&gt;\">limelight</p>", str
@@ -492,6 +453,29 @@ class TagHelperTest < ActionView::TestCase
     end
   end
 
+  def test_token_list_and_class_names_returns_an_html_safe_string
+    assert_predicate token_list("a value"), :html_safe?
+    assert_predicate class_names("a value"), :html_safe?
+  end
+
+  def test_token_list_and_class_names_only_html_escape_once
+    [:token_list, :class_names].each do |helper_method|
+      helper = ->(*arguments) { public_send(helper_method, *arguments) }
+
+      tokens = %w[
+        click->controller#action1
+        click->controller#action2
+        click->controller#action3
+        click->controller#action4
+      ]
+
+      token_list = tokens.reduce(&helper)
+
+      assert_predicate token_list, :html_safe?
+      assert_equal tokens, (CGI.unescape_html(token_list)).split
+    end
+  end
+
   def test_content_tag_with_data_attributes
     assert_dom_equal '<p data-number="1" data-string="hello" data-string-with-quotes="double&quot;quote&quot;party&quot;">limelight</p>',
       content_tag("p", "limelight", data: { number: 1, string: "hello", string_with_quotes: 'double"quote"party"' })
@@ -531,7 +515,11 @@ class TagHelperTest < ActionView::TestCase
   end
 
   def test_tag_attributes_escapes_values
-    assert_not_includes "<script>alert()</script>", render_erb(<<~HTML.strip)
+    expected_output = <<~HTML.strip
+      <input type="text" xss="&quot;&gt;&lt;script&gt;alert()&lt;/script&gt;">
+    HTML
+
+    assert_equal expected_output, render_erb(<<~HTML.strip)
       <input type="text" <%= tag.attributes xss: '"><script>alert()</script>' %>>
     HTML
   end
