@@ -509,13 +509,25 @@ class EnqueuedJobsTest < ActiveJob::TestCase
     assert_enqueued_with(job: LoggingJob, queue: "default")
   end
 
+  def test_assert_enqueued_with_when_queue_name_is_symbol
+    assert_enqueued_with(job: LoggingJob, queue: :default) do
+      LoggingJob.set(wait_until: Date.tomorrow.noon).perform_later
+    end
+  end
+
+  def test_assert_no_enqueued_jobs_and_perform_now
+    assert_no_enqueued_jobs do
+      LoggingJob.perform_now(1, 2, 3, keyword: true)
+    end
+  end
+
   def test_assert_enqueued_with_returns
     job = assert_enqueued_with(job: LoggingJob) do
       LoggingJob.set(wait_until: 5.minutes.from_now).perform_later(1, 2, 3, keyword: true)
     end
 
     assert_instance_of LoggingJob, job
-    assert_in_delta 5.minutes.from_now, job.scheduled_at, 1
+    assert_in_delta 5.minutes.from_now.to_f, job.scheduled_at.to_f, 1
     assert_equal "default", job.queue_name
     assert_equal [1, 2, 3, { keyword: true }], job.arguments
   end
@@ -525,7 +537,7 @@ class EnqueuedJobsTest < ActiveJob::TestCase
     job = assert_enqueued_with(job: LoggingJob)
 
     assert_instance_of LoggingJob, job
-    assert_in_delta 5.minutes.from_now, job.scheduled_at, 1
+    assert_in_delta 5.minutes.from_now.to_f, job.scheduled_at.to_f, 1
     assert_equal "default", job.queue_name
     assert_equal [1, 2, 3, { keyword: true }], job.arguments
   end
@@ -548,7 +560,7 @@ class EnqueuedJobsTest < ActiveJob::TestCase
       end
     end
 
-    assert_match(/No enqueued job found with {:job=>NestedJob, :queue=>"low"}/, error.message)
+    assert_match("No enqueued job found with #{{ job: NestedJob, queue: "low" }}", error.message)
   end
 
   def test_assert_enqueued_with_with_no_block_failure
@@ -562,7 +574,7 @@ class EnqueuedJobsTest < ActiveJob::TestCase
       assert_enqueued_with(job: NestedJob, queue: "low")
     end
 
-    assert_match(/No enqueued job found with {:job=>NestedJob, :queue=>"low"}/, error.message)
+    assert_match("No enqueued job found with #{{ job: NestedJob, queue: "low" }}", error.message)
   end
 
   def test_assert_enqueued_with_args
@@ -700,8 +712,8 @@ class EnqueuedJobsTest < ActiveJob::TestCase
       end
     end
 
-    assert_match(/No enqueued job found with {:job=>HelloJob, :args=>\[#{wilma.inspect}\]}/, error.message)
-    assert_match(/Potential matches: {.*?:job=>HelloJob, :args=>\[#<Person.* @id="9">\], :queue=>"default".*?}/, error.message)
+    assert_match("No enqueued job found with #{{ job: HelloJob, args: [wilma] }}", error.message)
+    assert_match(/Potential matches:.*Person.* @id="9"/, error.message)
   end
 
   def test_show_jobs_that_are_enqueued_when_job_is_not_queued_at_all
@@ -714,7 +726,7 @@ class EnqueuedJobsTest < ActiveJob::TestCase
       end
     end
 
-    assert_match(/No enqueued job found with {:job=>MultipleKwargsJob, :args=>\[#{wilma.inspect}\]}/, error.message)
+    assert_match("No enqueued job found with #{{ job: MultipleKwargsJob, args: [wilma] }}", error.message)
     assert_match(/No jobs of class MultipleKwargsJob were enqueued, job classes enqueued: HelloJob/, error.message)
   end
 
@@ -724,7 +736,7 @@ class EnqueuedJobsTest < ActiveJob::TestCase
       end
     end
 
-    assert_match(/No enqueued job found with {:job=>HelloJob, :args=>\[\]}/, error.message)
+    assert_match("No enqueued job found with #{{ job: HelloJob, args: [] }}", error.message)
     assert_match(/No jobs were enqueued/, error.message)
   end
 
@@ -736,8 +748,8 @@ class EnqueuedJobsTest < ActiveJob::TestCase
       assert_enqueued_with(job: HelloJob, args: [wilma])
     end
 
-    assert_match(/No enqueued job found with {:job=>HelloJob, :args=>\[#{wilma.inspect}\]}/, error.message)
-    assert_match(/Potential matches: {.*?:job=>HelloJob, :args=>\[#<Person.* @id="9">\], :queue=>"default".*?}/, error.message)
+    assert_match("No enqueued job found with #{{ job: HelloJob, args: [wilma] }}", error.message)
+    assert_match(/Potential matches: {.*?HelloJob,.*\[#<Person.* @id="9">\],.*?}/, error.message)
   end
 
   def test_assert_enqueued_with_does_not_change_jobs_count
@@ -1779,6 +1791,12 @@ class PerformedJobsTest < ActiveJob::TestCase
     assert_performed_with(job: NestedJob, queue: "default")
   end
 
+  def test_assert_performed_with_when_queue_name_is_symbol
+    assert_performed_with(job: NestedJob, queue: :default) do
+      NestedJob.perform_later
+    end
+  end
+
   def test_assert_performed_with_returns
     job = assert_performed_with(job: LoggingJob, queue: "default") do
       LoggingJob.perform_later(keyword: :sym)
@@ -1963,8 +1981,8 @@ class PerformedJobsTest < ActiveJob::TestCase
         HelloJob.perform_later(ricardo)
       end
     end
-    assert_match(/No performed job found with {:job=>HelloJob, :args=>\[#{wilma.inspect}\]}/, error.message)
-    assert_match(/Potential matches: {.*?:job=>HelloJob, :args=>\[#<Person.* @id="9">\], :queue=>"default".*?}/, error.message)
+    assert_match("No performed job found with #{{ job: HelloJob, args: [wilma] }}", error.message)
+    assert_match(/Potential matches: {.*?HelloJob,.*\[#<Person.* @id="9">\],.*?}/, error.message)
   end
 
   def test_assert_performed_with_without_block_failure_with_global_id_args
@@ -1976,8 +1994,8 @@ class PerformedJobsTest < ActiveJob::TestCase
       assert_performed_with(job: HelloJob, args: [wilma])
     end
 
-    assert_match(/No performed job found with {:job=>HelloJob, :args=>\[#{wilma.inspect}\]}/, error.message)
-    assert_match(/Potential matches: {.*?:job=>HelloJob, :args=>\[#<Person.* @id="9">\], :queue=>"default".*?}/, error.message)
+    assert_match("No performed job found with #{{ job: HelloJob, args: [wilma] }}", error.message)
+    assert_match(/Potential matches: {.*?HelloJob,.*\[#<Person.* @id="9">\],.*?}/, error.message)
   end
 
   def test_assert_performed_says_no_jobs_performed
@@ -1985,7 +2003,7 @@ class PerformedJobsTest < ActiveJob::TestCase
       assert_performed_with(job: HelloJob, args: [])
     end
 
-    assert_match(/No performed job found with {:job=>HelloJob, :args=>\[\]}/, error.message)
+    assert_match("No performed job found with #{{ job: HelloJob, args: [] }}", error.message)
     assert_match(/No jobs were performed/, error.message)
   end
 
@@ -1998,7 +2016,7 @@ class PerformedJobsTest < ActiveJob::TestCase
       assert_performed_with(job: MultipleKwargsJob, args: [wilma])
     end
 
-    assert_match(/No performed job found with {:job=>MultipleKwargsJob, :args=>\[#<Person.* @id=11>\]}/, error.message)
+    assert_match(/No performed job found with .*MultipleKwargsJob.*\[#<Person.* @id=11>\]}/, error.message)
     assert_match(/No jobs of class MultipleKwargsJob were performed, job classes performed: HelloJob/, error.message)
   end
 
@@ -2047,6 +2065,20 @@ class PerformedJobsTest < ActiveJob::TestCase
   end
 end
 
+class AdapterIsNotTestAdapterTest < ActiveJob::TestCase
+  def queue_adapter_for_test
+    ActiveJob::QueueAdapters::InlineAdapter.new
+  end
+
+  def test_perform_enqueued_jobs_just_yields
+    JobBuffer.clear
+    perform_enqueued_jobs do
+      HelloJob.perform_later("kevin")
+    end
+    assert_equal(1, JobBuffer.values.size)
+  end
+end
+
 class OverrideQueueAdapterTest < ActiveJob::TestCase
   class CustomQueueAdapter < ActiveJob::QueueAdapters::TestAdapter; end
 
