@@ -26,6 +26,7 @@ require "models/contract"
 require "models/subscription"
 require "models/book"
 require "models/branch"
+require "models/cpk"
 
 class AutomaticInverseFindingTests < ActiveRecord::TestCase
   fixtures :ratings, :comments, :cars, :books
@@ -602,6 +603,15 @@ class InverseHasManyTests < ActiveRecord::TestCase
     end
   end
 
+  def test_inverse_should_be_set_on_composite_primary_key_child
+    author = Cpk::Author.new(name: "John")
+    book = author.books.build(id: [nil, 1], title: "The Rails Way")
+    Cpk::Order.new(book: book, status: "paid")
+    author.save!
+
+    assert book.association(:order).loaded?
+  end
+
   def test_raise_record_not_found_error_when_invalid_ids_are_passed
     # delete all interest records to ensure that hard coded invalid_id(s)
     # are indeed invalid.
@@ -621,8 +631,8 @@ class InverseHasManyTests < ActiveRecord::TestCase
 
     exception = assert_raise(ActiveRecord::RecordNotFound) { human.interests.load.find() }
 
-    assert_equal exception.model, "Interest"
-    assert_equal exception.primary_key, "id"
+    assert_equal "Interest", exception.model
+    assert_equal "id", exception.primary_key
   end
 
   def test_trying_to_use_inverses_that_dont_exist_should_raise_an_error
@@ -783,7 +793,7 @@ class InverseBelongsToTests < ActiveRecord::TestCase
     end
   end
 
-  def test_with_hash_many_inversing_does_not_add_duplicate_associated_objects
+  def test_with_has_many_inversing_does_not_add_duplicate_associated_objects
     with_has_many_inversing(Interest) do
       human = Human.new
       interest = Interest.new(human: human)
