@@ -238,8 +238,14 @@ if ActiveRecord::Base.connection.prepared_statements
           #
           #   SELECT `authors`.* FROM `authors` WHERE `authors`.`id` IN (1, 2, 3)
           #
-          sql = "SELECT #{table}.* FROM #{table} WHERE #{pk} IN (#{bind_params(1..3)})"
+          params = if current_adapter?(:Mysql2Adapter, :TrilogyAdapter)
+            # With MySQL integers are casted as string for security.
+            bind_params((1..3).map(&:to_s))
+          else
+            bind_params(1..3)
+          end
 
+          sql = "SELECT #{table}.* FROM #{table} WHERE #{pk} IN (#{params})"
           arel_node = Arel.sql("SELECT #{table}.* FROM #{table} WHERE #{pk} IN (?)", [1, 2, 3])
           assert_equal sql, @connection.to_sql(arel_node)
           assert_sql(sql) { assert_equal 3, @connection.select_all(arel_node).length }
