@@ -941,34 +941,47 @@ class DirtyTest < ActiveRecord::TestCase
     assert_not_predicate person, :changed?
   end
 
-  unless current_adapter?(:SQLite3Adapter)
-    test "partial insert off with unchanged default function attribute" do
-      with_partial_writes Aircraft, false do
-        aircraft = Aircraft.new(name: "Boeing")
-        assert_equal "Boeing", aircraft.name
+  test "partial insert off with unchanged default function attribute" do
+    with_partial_writes Aircraft, false do
+      aircraft = Aircraft.new(name: "Boeing")
+      assert_equal "Boeing", aircraft.name
 
-        aircraft.save!
-        expected_manufactured_at = Time.now
-        aircraft.reload
+      time_before_saving = Time.now
+      aircraft.save!
+      time_after_saving = Time.now
+      aircraft.reload
 
-        assert_equal "Boeing", aircraft.name
-        assert_in_delta expected_manufactured_at, aircraft.manufactured_at, 1
-      end
+      assert_equal "Boeing", aircraft.name
+      assert_includes time_before_saving - 1..time_after_saving + 1, aircraft.manufactured_at
     end
+  end
 
-    test "partial insert off with changed default function attribute" do
-      with_partial_writes Aircraft, false do
-        manufactured_at = 1.years.ago
-        aircraft = Aircraft.new(name: "Boeing2", manufactured_at: manufactured_at)
+  test "partial insert off with changed default function attribute" do
+    with_partial_writes Aircraft, false do
+      manufactured_at = 1.years.ago
+      aircraft = Aircraft.new(name: "Boeing2", manufactured_at: manufactured_at)
 
-        assert_equal "Boeing2", aircraft.name
-        assert_equal manufactured_at.to_i, aircraft.manufactured_at.to_i
+      assert_equal "Boeing2", aircraft.name
+      assert_equal manufactured_at.to_i, aircraft.manufactured_at.to_i
 
-        aircraft.save!
-        aircraft.reload
+      aircraft.save!
+      aircraft.reload
 
-        assert_equal "Boeing2", aircraft.name
-        assert_equal manufactured_at.utc.strftime("%Y-%m-%d %H:%M:%S"), aircraft.manufactured_at.strftime("%Y-%m-%d %H:%M:%S")
+      assert_equal "Boeing2", aircraft.name
+      assert_equal manufactured_at.utc.strftime("%Y-%m-%d %H:%M:%S"), aircraft.manufactured_at.strftime("%Y-%m-%d %H:%M:%S")
+    end
+  end
+
+  if current_adapter?(:PostgreSQLAdapter) && ActiveRecord::Base.connection.supports_identity_columns?
+    test "partial insert off with changed composite identity primary key attribute" do
+      klass = Class.new(ActiveRecord::Base) do
+        self.table_name = "cpk_postgresql_identity_table"
+      end
+
+      with_partial_writes(klass, false) do
+        record = klass.create!(another_id: 10)
+        assert_equal 10, record.another_id
+        assert_not_nil record.id
       end
     end
   end
