@@ -60,12 +60,14 @@ class MigrationTest < ActiveRecord::TestCase
       Thing.connection.drop_table(table) rescue nil
     end
     Thing.reset_column_information
+    clear_statement_cache(Thing)
 
     %w(reminders people_reminders prefix_reminders_suffix).each do |table|
       Reminder.connection.drop_table(table) rescue nil
     end
     Reminder.reset_table_name
     Reminder.reset_column_information
+    clear_statement_cache(Reminder)
 
     %w(last_name key bio age height wealth birthday favorite_day
        moment_of_truth male administrator funny).each do |column|
@@ -75,6 +77,7 @@ class MigrationTest < ActiveRecord::TestCase
     Person.connection.remove_column("people", "middle_name") rescue nil
     Person.connection.add_column("people", "first_name", :string)
     Person.reset_column_information
+    clear_statement_cache(Person)
 
     ActiveRecord::Migration.verbose = @verbose_was
   end
@@ -1136,6 +1139,12 @@ class MigrationTest < ActiveRecord::TestCase
       }
     end
 
+    def clear_statement_cache(model)
+      model.connection_handler.each_connection_pool do |pool|
+        pool.connections.each(&:clear_cache!)
+      end
+    end
+
     def with_another_process_holding_lock(lock_id)
       thread_lock = Concurrent::CountDownLatch.new
       test_terminated = Concurrent::CountDownLatch.new
