@@ -8,6 +8,12 @@ require "models/post"
 class SerializedAttributeTest < ActiveRecord::TestCase
   def setup
     ActiveRecord.use_yaml_unsafe_load = true
+    @yaml_column_permitted_classes_default = ActiveRecord.yaml_column_permitted_classes
+  end
+
+  def teardown
+    Topic.serialize("content")
+    ActiveRecord.yaml_column_permitted_classes = @yaml_column_permitted_classes_default
   end
 
   fixtures :topics, :posts
@@ -19,11 +25,11 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   class ImportantTopic < Topic
-    serialize :important, Hash
+    serialize :important, type: Hash
   end
 
-  teardown do
-    Topic.serialize("content")
+  class ClassifiedTopic < Topic
+    serialize :important, type: Class
   end
 
   def test_serialize_does_not_eagerly_load_columns
@@ -34,7 +40,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_serialized_attribute
-    Topic.serialize("content", MyObject)
+    Topic.serialize("content", type: MyObject)
 
     myobj = MyObject.new("value1", "value2")
     topic = Topic.create("content" => myobj)
@@ -48,7 +54,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     klass = Class.new(ActiveRecord::Base) do
       self.table_name = Topic.table_name
       alias_attribute :object, :content
-      serialize :object, MyObject
+      serialize :object, type: MyObject
     end
 
     myobj = MyObject.new("value1", "value2")
@@ -62,7 +68,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   def test_serialized_attribute_with_default
     klass = Class.new(ActiveRecord::Base) do
       self.table_name = Topic.table_name
-      serialize(:content, Hash, default: { key: "value" })
+      serialize(:content, type: Hash, default: { key: "value" })
     end
 
     t = klass.new
@@ -73,7 +79,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     klass = Class.new(ActiveRecord::Base) do
       self.table_name = Topic.table_name
       attribute :content, default: { key: "value" }
-      serialize :content, Hash
+      serialize :content, type: Hash
     end
 
     t = klass.new
@@ -81,7 +87,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_serialized_attribute_in_base_class
-    Topic.serialize("content", Hash)
+    Topic.serialize("content", type: Hash)
 
     hash = { "content1" => "value1", "content2" => "value2" }
     important_topic = ImportantTopic.create("content" => hash)
@@ -92,7 +98,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_serialized_attributes_from_database_on_subclass
-    Topic.serialize :content, Hash
+    Topic.serialize :content, type: Hash
 
     t = ImportantTopic.new(content: { foo: :bar })
     assert_equal({ foo: :bar }, t.content)
@@ -102,7 +108,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_serialized_attribute_calling_dup_method
-    Topic.serialize :content, JSON
+    Topic.serialize :content, coder: JSON
 
     orig = Topic.new(content: { foo: :bar })
     clone = orig.dup
@@ -110,7 +116,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_serialized_json_attribute_returns_unserialized_value
-    Topic.serialize :content, JSON
+    Topic.serialize :content, coder: JSON
     my_post = posts(:welcome)
 
     t = Topic.new(content: my_post)
@@ -123,7 +129,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_json_read_legacy_null
-    Topic.serialize :content, JSON
+    Topic.serialize :content, coder: JSON
 
     # Force a row to have a JSON "null" instead of a database NULL (this is how
     # null values are saved on 4.1 and before)
@@ -134,7 +140,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_json_read_db_null
-    Topic.serialize :content, JSON
+    Topic.serialize :content, coder: JSON
 
     # Force a row to have a database NULL instead of a JSON "null"
     id = Topic.connection.insert "INSERT INTO topics (content) VALUES(NULL)"
@@ -165,6 +171,23 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     assert_equal(myobj, topic.content)
   end
 
+  def test_serialized_class_attribute
+    ActiveRecord.yaml_column_permitted_classes += [Class]
+
+    topic = ClassifiedTopic.create(important: Symbol).reload
+    assert_equal(Symbol, topic.important)
+    assert_not_empty ClassifiedTopic.where(important: Symbol)
+  end
+
+  def test_serialized_class_does_not_become_frozen
+    ActiveRecord.yaml_column_permitted_classes += [Class]
+
+    assert_not_predicate Symbol, :frozen?
+    ClassifiedTopic.create(important: Symbol)
+    assert_not_empty ClassifiedTopic.where(important: Symbol)
+    assert_not_predicate Symbol, :frozen?
+  end
+
   def test_nil_serialized_attribute_without_class_constraint
     topic = Topic.new
     assert_nil topic.content
@@ -176,13 +199,13 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_nil_not_serialized_with_class_constraint
-    Topic.serialize :content, Hash
+    Topic.serialize :content, type: Hash
     assert Topic.new(content: nil).save
     assert_equal 1, Topic.where(content: nil).count
   end
 
   def test_serialized_attribute_should_raise_exception_on_assignment_with_wrong_type
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     assert_raise(ActiveRecord::SerializationTypeMismatch) do
       Topic.new(content: "string")
     end
@@ -192,13 +215,13 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     myobj = MyObject.new("value1", "value2")
     topic = Topic.new(content: myobj)
     assert topic.save
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     assert_raise(ActiveRecord::SerializationTypeMismatch) { Topic.find(topic.id).content }
   end
 
   def test_serialized_attribute_with_class_constraint
     settings = { "color" => "blue" }
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     topic = Topic.new(content: settings)
     assert topic.save
     assert_equal(settings, Topic.find(topic.id).content)
@@ -206,27 +229,27 @@ class SerializedAttributeTest < ActiveRecord::TestCase
 
   def test_where_by_serialized_attribute_with_array
     settings = [ "color" => "green" ]
-    Topic.serialize(:content, Array)
+    Topic.serialize(:content, type: Array)
     topic = Topic.create!(content: settings)
     assert_equal topic, Topic.where(content: settings).take
   end
 
   def test_where_by_serialized_attribute_with_hash
     settings = { "color" => "green" }
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     topic = Topic.create!(content: settings)
     assert_equal topic, Topic.where(content: settings).take
   end
 
   def test_where_by_serialized_attribute_with_hash_in_array
     settings = { "color" => "green" }
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     topic = Topic.create!(content: settings)
-    assert_equal topic, Topic.where(content: [settings]).take
+    assert_equal topic, Topic.where(content: [settings, { "herring" => "red" }]).take
   end
 
   def test_serialized_default_class
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     topic = Topic.new
     assert_equal Hash, topic.content.class
     assert_equal Hash, topic.read_attribute(:content).class
@@ -267,7 +290,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
       end
     end
 
-    Topic.serialize(:content, some_class)
+    Topic.serialize(:content, coder: some_class)
     topic = Topic.new(content: some_class.new("my value"))
     topic.save!
     topic.reload
@@ -277,7 +300,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
 
   def test_serialize_attribute_via_select_method_when_time_zone_available
     with_timezone_config aware_attributes: true do
-      Topic.serialize(:content, MyObject)
+      Topic.serialize(:content, type: MyObject)
 
       myobj = MyObject.new("value1", "value2")
       topic = Topic.create(content: myobj)
@@ -302,16 +325,16 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_unexpected_serialized_type
-    Topic.serialize :content, Hash
+    Topic.serialize :content, type: Hash
     topic = Topic.create!(content: { zomg: true })
 
-    Topic.serialize :content, Array
+    Topic.serialize :content, type: Array
 
     topic.reload
     error = assert_raise(ActiveRecord::SerializationTypeMismatch) do
       topic.content
     end
-    expected = "can't load `content`: was supposed to be a Array, but was a Hash. -- {:zomg=>true}"
+    expected = "can't load `content`: was supposed to be a Array, but was a Hash. -- #{{ zomg: true }}"
     assert_equal expected, error.to_s
   end
 
@@ -334,7 +357,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_nil_is_not_changed_when_serialized_with_a_class
-    Topic.serialize(:content, Array)
+    Topic.serialize(:content, type: Array)
 
     topic = Topic.new(content: nil)
 
@@ -343,12 +366,12 @@ class SerializedAttributeTest < ActiveRecord::TestCase
 
   def test_classes_without_no_arg_constructors_are_not_supported
     assert_raises(ArgumentError) do
-      Topic.serialize(:content, Regexp)
+      Topic.serialize(:content, type: Regexp)
     end
   end
 
   def test_newly_emptied_serialized_hash_is_changed
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     topic = Topic.create(content: { "things" => "stuff" })
     topic.content.delete("things")
     topic.save!
@@ -396,15 +419,29 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     #
     # model.attribute = value
     # assert_equal model.attribute, model.tap(&:save).reload.attribute
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
     topic = Topic.create!(content: {})
     topic2 = Topic.create!(content: nil)
 
     assert_equal [topic, topic2], Topic.where(content: nil).sort_by(&:id)
   end
 
+  def test_serialized_attribute_can_be_defined_in_abstract_classes
+    klass = Class.new(ActiveRecord::Base) do
+      self.abstract_class = true
+      self.table_name = nil
+      serialize(:content, type: Hash)
+    end
+
+    subclass = Class.new(klass) do
+      self.table_name = "posts"
+    end
+
+    subclass.define_attribute_methods
+  end
+
   def test_nil_is_always_persisted_as_null
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
 
     topic = Topic.create!(content: { foo: "bar" })
     topic.update_attribute :content, nil
@@ -498,7 +535,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     end.new
     model = Class.new(Topic) do
       attribute :foo, type
-      serialize :foo, coder
+      serialize :foo, coder: coder
     end
 
     topic = model.create!(foo: "bar")
@@ -512,7 +549,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     topic = model.create!
     topic.update group: "1"
 
-    model.serialize :group, JSON
+    model.serialize :group, coder: JSON
     model.reset_column_information
 
     # This isn't strictly necessary for the test, but a little bit of
@@ -537,11 +574,19 @@ end
 
 class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
   def setup
+    @use_yaml_unsafe_load = ActiveRecord.use_yaml_unsafe_load
+    @yaml_column_permitted_classes_default = ActiveRecord.yaml_column_permitted_classes
     ActiveRecord.use_yaml_unsafe_load = false
   end
 
+  def teardown
+    Topic.serialize("content")
+    ActiveRecord.yaml_column_permitted_classes = @yaml_column_permitted_classes_default
+    ActiveRecord.use_yaml_unsafe_load = @use_yaml_unsafe_load
+  end
+
   def test_serialized_attribute
-    Topic.serialize("content", String)
+    Topic.serialize("content", type: String)
 
     myobj = String.new("value1")
     topic = Topic.create("content" => myobj)
@@ -555,7 +600,7 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
     klass = Class.new(ActiveRecord::Base) do
       self.table_name = Topic.table_name
       attribute :content, default: { "key" => "value" }
-      serialize :content, Hash
+      serialize :content, type: Hash
     end
 
     t = klass.new
@@ -563,7 +608,7 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
   end
 
   def test_nil_is_always_persisted_as_null
-    Topic.serialize(:content, Hash)
+    Topic.serialize(:content, type: Hash)
 
     topic = Topic.create!(content: { "foo" => "bar" })
     topic.update_attribute :content, nil
@@ -573,7 +618,7 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
   def test_serialized_attribute_with_default
     klass = Class.new(ActiveRecord::Base) do
       self.table_name = Topic.table_name
-      serialize(:content, Hash, default: { "key" => "value" })
+      serialize(:content, type: Hash, default: { "key" => "value" })
     end
 
     t = klass.new
@@ -581,7 +626,7 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
   end
 
   def test_serialized_attributes_from_database_on_subclass
-    Topic.serialize :content, Hash
+    Topic.serialize :content, type: Hash
 
     t = ImportantTopic.new(content: { "foo" => "bar" })
     assert_equal({ "foo" => "bar" }, t.content)
@@ -594,7 +639,7 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
     klass = Class.new(ActiveRecord::Base) do
       self.table_name = Topic.table_name
       alias_attribute :object, :content
-      serialize :object, Hash
+      serialize :object, type: Hash
     end
 
     myobj = { "somevalue" => "thevalue" }
@@ -606,22 +651,22 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
   end
 
   def test_unexpected_serialized_type
-    Topic.serialize :content, Hash
+    Topic.serialize :content, type: Hash
     topic = Topic.create!(content: { "zomg" => true })
 
-    Topic.serialize :content, Array
+    Topic.serialize :content, type: Array
 
     topic.reload
     error = assert_raise(ActiveRecord::SerializationTypeMismatch) do
       topic.content
     end
-    expected = "can't load `content`: was supposed to be a Array, but was a Hash. -- {\"zomg\"=>true}"
+    expected = "can't load `content`: was supposed to be a Array, but was a Hash. -- #{{ "zomg" => true }}"
     assert_equal expected, error.to_s
   end
 
   def test_serialize_attribute_via_select_method_when_time_zone_available
     with_timezone_config aware_attributes: true do
-      Topic.serialize(:content, Hash)
+      Topic.serialize(:content, type: Hash)
 
       myobj = { "somevalue" => "thevalue" }
       topic = Topic.create(content: myobj)
@@ -635,11 +680,39 @@ class SerializedAttributeTestWithYamlSafeLoad < SerializedAttributeTest
     myobj = { "somevalue" => "thevalue" }
     topic = Topic.new(content: myobj)
     assert topic.save
-    Topic.serialize(:content, String)
+    Topic.serialize(:content, type: String)
     assert_raise(ActiveRecord::SerializationTypeMismatch) { Topic.find(topic.id).content }
   end
 
   def test_serialized_time_attribute
     skip "Time is a DisallowedClass in Psych safe_load()."
   end
+
+  def test_supports_permitted_classes_for_default_column_serializer
+    Topic.serialize(:content, yaml: { permitted_classes: [Time] })
+    topic = Topic.new(content: Time.now)
+    assert topic.save
+  end
+
+  def test_recognizes_coder_as_deprecated_positional_argument
+    some_coder = Struct.new(:foo) do
+      def self.dump(value)
+        value.foo
+      end
+
+      def self.load(value)
+        new(value)
+      end
+    end
+
+    assert_deprecated(/Please pass the coder as a keyword argument/, ActiveRecord.deprecator) do
+      Topic.serialize(:content, some_coder)
+    end
+  end
+
+  def test_recognizes_type_as_deprecated_positional_argument
+    assert_deprecated(/Please pass the class as a keyword argument/, ActiveRecord.deprecator) do
+      Topic.serialize(:content, Hash)
+    end
+  end
 end
