@@ -23,10 +23,10 @@ class Comment < ActiveRecord::Base
   belongs_to :first_post, foreign_key: :post_id
   belongs_to :special_post_with_default_scope, foreign_key: :post_id
 
-  has_many :children, class_name: "Comment", foreign_key: :parent_id, inverse_of: :parent
-  belongs_to :parent, class_name: "Comment", counter_cache: :children_count, inverse_of: :children
+  has_one :post_with_inverse, ->(comment) { where(id: comment.post_id) }, class_name: "FirstPost", inverse_of: :comment_with_inverse
 
-  alias_attribute :entry, :post
+  has_many :children, class_name: "Comment", inverse_of: :parent
+  belongs_to :parent, class_name: "Comment", counter_cache: :children_count, inverse_of: :children
 
   enum label: [:default, :child]
 
