@@ -20,7 +20,7 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     assert_equal "town.jpg", @user.highlights.second.filename.to_s
 
     assert_not_empty @user.highlights_attachments
-    assert_equal @user.highlights_blobs.count, 2
+    assert_equal 2, @user.highlights_blobs.count
   end
 
   test "attaching existing blobs from signed IDs to an existing record" do
@@ -132,6 +132,83 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     assert ActiveStorage::Blob.service.exist?(@user.highlights.second.key)
   end
 
+  test "attaching new blobs within a transaction uploads all the files" do
+    @user.highlights.attach fixture_file_upload("image.gif")
+
+    ActiveRecord::Base.transaction do
+      @user.highlights.attach fixture_file_upload("racecar.jpg")
+      @user.highlights.attach fixture_file_upload("video.mp4")
+    end
+
+    assert_equal "image.gif", @user.highlights.first.filename.to_s
+    assert_equal "racecar.jpg", @user.highlights.second.filename.to_s
+    assert_equal "video.mp4", @user.highlights.third.filename.to_s
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.first.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.second.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.third.key)
+  end
+
+  test "attaching many new blobs within a transaction uploads all the files" do
+    ActiveRecord::Base.transaction do
+      @user.highlights.attach [fixture_file_upload("image.gif"), fixture_file_upload("racecar.jpg")]
+      @user.highlights.attach fixture_file_upload("video.mp4")
+    end
+
+    assert_equal "image.gif", @user.highlights.first.filename.to_s
+    assert_equal "racecar.jpg", @user.highlights.second.filename.to_s
+    assert_equal "video.mp4", @user.highlights.third.filename.to_s
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.first.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.second.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.third.key)
+  end
+
+  test "attaching many new blobs within a transaction on a dirty record uploads all the files" do
+    @user.name = "Tina"
+
+    ActiveRecord::Base.transaction do
+      @user.highlights.attach fixture_file_upload("image.gif")
+      @user.highlights.attach fixture_file_upload("racecar.jpg")
+    end
+
+    @user.highlights.attach fixture_file_upload("video.mp4")
+    @user.save
+
+    assert_equal "image.gif", @user.highlights.first.filename.to_s
+    assert_equal "racecar.jpg", @user.highlights.second.filename.to_s
+    assert_equal "video.mp4", @user.highlights.third.filename.to_s
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.first.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.second.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.third.key)
+  end
+
+  test "attaching many new blobs within a transaction on a new record uploads all the files" do
+    user = User.create!(name: "John") do |user|
+      user.highlights.attach(io: StringIO.new("STUFF"), filename: "funky.jpg", content_type: "image/jpeg")
+      user.highlights.attach(io: StringIO.new("THINGS"), filename: "town.jpg", content_type: "image/jpeg")
+    end
+
+    assert_equal 2, user.highlights.count
+    assert_equal "funky.jpg", user.highlights.first.filename.to_s
+    assert_equal "town.jpg", user.highlights.second.filename.to_s
+    assert ActiveStorage::Blob.service.exist?(user.highlights.first.key)
+    assert ActiveStorage::Blob.service.exist?(user.highlights.second.key)
+  end
+
+  test "attaching new blobs within a transaction create the exact amount of records" do
+    assert_difference -> { ActiveStorage::Blob.count }, +2 do
+      ActiveRecord::Base.transaction do
+        @user.highlights.attach fixture_file_upload("racecar.jpg")
+        @user.highlights.attach fixture_file_upload("video.mp4")
+      end
+    end
+
+    assert_equal 2, @user.highlights.count
+    assert_equal "racecar.jpg", @user.highlights.first.filename.to_s
+    assert_equal "video.mp4", @user.highlights.second.filename.to_s
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.first.key)
+    assert ActiveStorage::Blob.service.exist?(@user.highlights.second.key)
+  end
+
   test "attaching existing blobs to an existing record one at a time" do
     @user.highlights.attach create_blob(filename: "funky.jpg")
     @user.highlights.attach create_blob(filename: "town.jpg")
@@ -308,23 +385,19 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     end
   end
 
-  test "updating an existing record with attachments when appending on assign" do
-    append_on_assign do
-      assert_deprecated do
-        @user.highlights.attach create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg")
+  test "updating an existing record with attachments" do
+    @user.highlights.attach create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg")
 
-        assert_difference -> { @user.reload.highlights.count }, +2 do
-          @user.update! highlights: [ create_blob(filename: "whenever.jpg"), create_blob(filename: "wherever.jpg") ]
-        end
+    assert_difference -> { @user.reload.highlights.count }, -2 do
+      @user.update! highlights: []
+    end
 
-        assert_no_difference -> { @user.reload.highlights.count } do
-          @user.update! highlights: [ ]
-        end
+    assert_difference -> { @user.reload.highlights.count }, 2 do
+      @user.update! highlights: [ create_blob(filename: "whenever.jpg"), create_blob(filename: "wherever.jpg") ]
+    end
 
-        assert_no_difference -> { @user.reload.highlights.count } do
-          @user.update! highlights: nil
-        end
-      end
+    assert_difference -> { @user.reload.highlights.count }, -2 do
+      @user.update! highlights: nil
     end
   end
 
@@ -492,28 +565,6 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     end
   end
 
-  test "purging from the attachments relation" do
-    [ create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg") ].tap do |blobs|
-      @user.highlights.attach blobs
-      assert @user.highlights.attached?
-
-      message = <<-MSG.squish
-        Calling `purge` from `highlights_attachments` is deprecated and will be removed in Rails 7.1.
-        To migrate to Rails 7.1's behavior call `purge` from `highlights` instead: `highlights.purge`.
-      MSG
-      assert_deprecated(message) do
-        assert_changes -> { @user.updated_at } do
-          @user.highlights_attachments.purge
-        end
-      end
-      assert_not @user.highlights.attached?
-      assert_not ActiveStorage::Blob.exists?(blobs.first.id)
-      assert_not ActiveStorage::Blob.exists?(blobs.second.id)
-      assert_not ActiveStorage::Blob.service.exist?(blobs.first.key)
-      assert_not ActiveStorage::Blob.service.exist?(blobs.second.key)
-    end
-  end
-
   test "purging attachment with shared blobs" do
     [
       create_blob(filename: "funky.jpg"),
@@ -587,31 +638,6 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     end
   end
 
-  test "purging later from the attachments relation" do
-    [ create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg") ].tap do |blobs|
-      @user.highlights.attach blobs
-      assert @user.highlights.attached?
-
-      message = <<-MSG.squish
-        Calling `purge_later` from `highlights_attachments` is deprecated and will be removed in Rails 7.1.
-        To migrate to Rails 7.1's behavior call `purge_later` from `highlights` instead: `highlights.purge_later`.
-      MSG
-      assert_deprecated(message) do
-        perform_enqueued_jobs do
-          assert_changes -> { @user.updated_at } do
-            @user.highlights_attachments.purge_later
-          end
-        end
-      end
-
-      assert_not @user.highlights.attached?
-      assert_not ActiveStorage::Blob.exists?(blobs.first.id)
-      assert_not ActiveStorage::Blob.exists?(blobs.second.id)
-      assert_not ActiveStorage::Blob.service.exist?(blobs.first.key)
-      assert_not ActiveStorage::Blob.service.exist?(blobs.second.key)
-    end
-  end
-
   test "purging attachment later with shared blobs" do
     [
       create_blob(filename: "funky.jpg"),
@@ -746,6 +772,47 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     end
   end
 
+  test "attaching blobs to a persisted, unchanged, and valid record, returns the attachments" do
+    @user.highlights.attach create_blob(filename: "racecar.jpg")
+    return_value = @user.highlights.attach create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg")
+    assert_equal @user.highlights, return_value
+  end
+
+  test "attaching blobs to a persisted, unchanged, and invalid record, returns nil" do
+    @user.update_attribute(:name, nil)
+    assert_not @user.valid?
+
+    @user.highlights.attach create_blob(filename: "racecar.jpg")
+    return_value = @user.highlights.attach create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg")
+    assert_nil return_value
+  end
+
+  test "attaching blobs to a changed record, returns the attachments" do
+    @user.name = "Tina"
+    @user.highlights.attach create_blob(filename: "racecar.jpg")
+    return_value = @user.highlights.attach create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg")
+    assert_equal @user.highlights, return_value
+  end
+
+  test "attaching blobs to a non persisted record, returns the attachments" do
+    user = User.new(name: "John")
+    user.highlights.attach create_blob(filename: "racecar.jpg")
+    return_value = user.highlights.attach create_blob(filename: "funky.jpg"), create_blob(filename: "town.jpg")
+    assert_equal user.highlights, return_value
+  end
+
+  test "raises error when global service configuration is missing" do
+    Rails.configuration.active_storage.stub(:service, nil) do
+      error = assert_raises RuntimeError do
+        User.class_eval do
+          has_many_attached :featured_photos
+        end
+      end
+
+      assert_match(/Missing Active Storage service name. Specify Active Storage service name for config.active_storage.service in config\/environments\/test.rb/, error.message)
+    end
+  end
+
   test "raises error when misconfigured service is passed" do
     error = assert_raises ArgumentError do
       User.class_eval do
@@ -757,7 +824,9 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
   end
 
   test "creating variation by variation name" do
-    @user.highlights_with_variants.attach fixture_file_upload("racecar.jpg")
+    assert_no_enqueued_jobs only: ActiveStorage::TransformJob do
+      @user.highlights_with_variants.attach fixture_file_upload("racecar.jpg")
+    end
     variant = @user.highlights_with_variants.first.variant(:thumb).processed
 
     image = read_image(variant)
@@ -766,7 +835,7 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     assert_equal 67, image.height
   end
 
-  test "raises error when unknown variant name is used" do
+  test "raises error when unknown variant name is used to generate variant" do
     @user.highlights_with_variants.attach fixture_file_upload("racecar.jpg")
 
     error = assert_raises ArgumentError do
@@ -776,47 +845,102 @@ class ActiveStorage::ManyAttachedTest < ActiveSupport::TestCase
     assert_match(/Cannot find variant :unknown for User#highlights_with_variants/, error.message)
   end
 
+  test "creating preview by variation name" do
+    @user.highlights_with_variants.attach fixture_file_upload("report.pdf")
+    preview = @user.highlights_with_variants.first.preview(:thumb).processed
+
+    image = read_image(preview.send(:variant))
+    assert_equal "PNG", image.type
+    assert_equal 77, image.width
+    assert_equal 100, image.height
+  end
+
+  test "raises error when unknown variant name is used to generate preview" do
+    @user.highlights_with_variants.attach fixture_file_upload("report.pdf")
+
+    error = assert_raises ArgumentError do
+      @user.highlights_with_variants.first.preview(:unknown).processed
+    end
+
+    assert_match(/Cannot find variant :unknown for User#highlights_with_variants/, error.message)
+  end
+
+  test "creating representation by variation name" do
+    @user.highlights_with_variants.attach fixture_file_upload("racecar.jpg")
+    variant = @user.highlights_with_variants.first.representation(:thumb).processed
+
+    image = read_image(variant)
+    assert_equal "JPEG", image.type
+    assert_equal 100, image.width
+    assert_equal 67, image.height
+  end
+
+  test "raises error when unknown variant name is used to generate representation" do
+    @user.highlights_with_variants.attach fixture_file_upload("racecar.jpg")
+
+    error = assert_raises ArgumentError do
+      @user.highlights_with_variants.first.representation(:unknown).processed
+    end
+
+    assert_match(/Cannot find variant :unknown for User#highlights_with_variants/, error.message)
+  end
+
+  test "transforms variants later" do
+    blob = create_file_blob(filename: "racecar.jpg")
+
+    assert_enqueued_with job: ActiveStorage::TransformJob, args: [blob, resize_to_limit: [1, 1]] do
+      @user.highlights_with_preprocessed.attach blob
+    end
+  end
+
+  test "transforms variants later conditionally via proc" do
+    assert_no_enqueued_jobs only: ActiveStorage::TransformJob do
+      @user.highlights_with_conditional_preprocessed.attach create_file_blob(filename: "racecar.jpg")
+    end
+
+    blob = create_file_blob(filename: "racecar.jpg")
+    @user.update(name: "transform via proc")
+
+    assert_enqueued_with job: ActiveStorage::TransformJob, args: [blob, resize_to_limit: [2, 2]] do
+      @user.highlights_with_conditional_preprocessed.attach blob
+    end
+  end
+
+  test "transforms variants later conditionally via method" do
+    assert_no_enqueued_jobs only: ActiveStorage::TransformJob do
+      @user.highlights_with_conditional_preprocessed.attach create_file_blob(filename: "racecar.jpg")
+    end
+
+    blob = create_file_blob(filename: "racecar.jpg")
+    @user.update(name: "transform via method")
+
+    assert_enqueued_with job: ActiveStorage::TransformJob, args: [blob, resize_to_limit: [3, 3]] do
+      @user.highlights_with_conditional_preprocessed.attach blob
+    end
+  end
+
+  test "avoids enqueuing transform later job when blob is not representable" do
+    unrepresentable_blob = create_blob(filename: "hello.txt")
+
+    assert_no_enqueued_jobs only: ActiveStorage::TransformJob do
+      @user.highlights_with_preprocessed.attach unrepresentable_blob
+    end
+  end
+
   test "successfully attaches new blobs and destroys attachments marked for destruction via nested attributes" do
-    append_on_assign do
-      assert_deprecated do
-        town_blob = create_blob(filename: "town.jpg")
-        @user.highlights.attach(town_blob)
-        @user.reload
+    town_blob = create_blob(filename: "town.jpg")
+    @user.highlights.attach(town_blob)
+    @user.reload
 
-        racecar_blob = fixture_file_upload("racecar.jpg")
-        attachment_id = town_blob.attachments.find_by!(record: @user).id
-        @user.update(
-          highlights: [racecar_blob],
-          highlights_attachments_attributes: [{ id: attachment_id, _destroy: true }]
-        )
+    racecar_blob = fixture_file_upload("racecar.jpg")
+    attachment_id = town_blob.attachments.find_by!(record: @user).id
+    @user.update(
+      highlights: [racecar_blob],
+      highlights_attachments_attributes: [{ id: attachment_id, _destroy: true }]
+    )
 
-        assert @user.reload.highlights.attached?
-        assert_equal 1, @user.highlights.count
-        assert_equal "racecar.jpg", @user.highlights.blobs.first.filename.to_s
-      end
-    end
+    assert @user.reload.highlights.attached?
+    assert_equal 1, @user.highlights.count
+    assert_equal "racecar.jpg", @user.highlights.blobs.first.filename.to_s
   end
-
-  test "deprecation warning when replace_on_assign_to_many is false" do
-    append_on_assign do
-      message = <<-MSG.squish
-        DEPRECATION WARNING: config.active_storage.replace_on_assign_to_many is deprecated and will be removed in Rails 7.1.
-        Make sure that your code works well with config.active_storage.replace_on_assign_to_many set to true before upgrading.
-        To append new attachables to the Active Storage association, prefer using `attach`.
-        Using association setter would result in purging the existing attached attachments and replacing them with new ones.
-      MSG
-
-      assert_deprecated(message) do
-        @user.update! highlights: [create_blob(filename: "whenever.jpg")]
-      end
-    end
-  end
-
-  private
-    def append_on_assign
-      ActiveStorage.replace_on_assign_to_many, previous = false, ActiveStorage.replace_on_assign_to_many
-      yield
-    ensure
-      ActiveStorage.replace_on_assign_to_many = previous
-    end
 end
