@@ -108,31 +108,46 @@ module ApplicationTests
 
     test "rails/welcome in production" do
       app("production")
-      get "/"
+      get("/", {}, "HTTPS" => "on")
       assert_equal 404, last_response.status
     end
 
     test "rails/info in production" do
       app("production")
-      get "/rails/info"
+      get("/rails/info", {}, "HTTPS" => "on")
       assert_equal 404, last_response.status
     end
 
     test "rails/info/routes in production" do
       app("production")
-      get "/rails/info/routes"
+      get("/rails/info/routes", {}, "HTTPS" => "on")
       assert_equal 404, last_response.status
     end
 
     test "rails/info/properties in production" do
       app("production")
-      get "/rails/info/properties"
+      get("/rails/info/properties", {}, "HTTPS" => "on")
       assert_equal 404, last_response.status
     end
 
+    test "rails/health in production" do
+      app("production")
+
+      app_file "config/routes.rb", <<-RUBY
+        Rails.application.routes.draw do
+          get "up" => "rails/health#show", as: :rails_health_check
+        end
+      RUBY
+
+      get("/up", {}, "HTTPS" => "on")
+      assert_equal 200, last_response.status
+    end
+
     test "simple controller" do
       simple_controller
 
+      app "development"
+
       get "/foo"
       assert_equal "foo", last_response.body
     end
@@ -160,6 +175,8 @@ module ApplicationTests
         end
       RUBY
 
+      app "development"
+
       get "/foo"
       assert_equal "bar", last_response.body
     end
@@ -174,6 +191,8 @@ module ApplicationTests
         end
       RUBY
 
+      app "development"
+
       get "/blog/archives"
       assert_equal "/archives", last_response.body
     end
@@ -194,6 +213,8 @@ module ApplicationTests
         end
       RUBY
 
+      app "development"
+
       get "/foo"
       assert_equal "/blog", last_response.body
     end
@@ -221,6 +242,8 @@ module ApplicationTests
         end
       RUBY
 
+      app "development"
+
       get "/foo"
       assert_equal "foo", last_response.body
 
@@ -254,6 +277,8 @@ module ApplicationTests
         end
       RUBY
 
+      app "development"
+
       get "/foo"
       assert_equal "foo", last_response.body
 
@@ -370,13 +395,15 @@ module ApplicationTests
 
         app(mode)
 
-        get "/foo"
+        https = (mode == "production" ? "on" : "off")
+
+        get("/foo", {}, "HTTPS" => https)
         assert_equal "bar", last_response.body
 
-        get "/custom"
+        get("/custom", {}, "HTTPS" => https)
         assert_equal "http://www.microsoft.com", last_response.body
 
-        get "/mapping"
+        get("/mapping", {}, "HTTPS" => https)
         assert_equal "/profile", last_response.body
 
         app_file "config/routes.rb", <<-RUBY
@@ -396,13 +423,13 @@ module ApplicationTests
 
         sleep 0.1
 
-        get "/foo"
+        get("/foo", {}, "HTTPS" => https)
         assert_equal expected_action, last_response.body
 
-        get "/custom"
+        get("/custom", {}, "HTTPS" => https)
         assert_equal expected_url, last_response.body
 
-        get "/mapping"
+        get("/mapping", {}, "HTTPS" => https)
         assert_equal expected_mapping, last_response.body
       end
     end
@@ -650,6 +677,8 @@ module ApplicationTests
     end
 
     test "resource routing with irregular inflection" do
+      app("development")
+
       app_file "config/initializers/inflection.rb", <<-RUBY
         ActiveSupport::Inflector.inflections do |inflect|
           inflect.irregular 'yazi', 'yazilar'
@@ -678,6 +707,8 @@ module ApplicationTests
     end
 
     test "reloading routes removes methods and doesn't undefine them" do
+      app("development")
+
       app_file "config/routes.rb", <<-RUBY
         Rails.application.routes.draw do
           get '/url', to: 'url#index'
@@ -738,7 +769,7 @@ module ApplicationTests
     test "request to rails/welcome for api_only app is successful" do
       add_to_config <<-RUBY
         config.api_only = true
-        config.action_dispatch.show_exceptions = false
+        config.action_dispatch.show_exceptions = :none
         config.action_controller.allow_forgery_protection = true
       RUBY
 
@@ -750,7 +781,7 @@ module ApplicationTests
 
     test "request to rails/welcome is successful when default_protect_from_forgery is false" do
       add_to_config <<-RUBY
-        config.action_dispatch.show_exceptions = false
+        config.action_dispatch.show_exceptions = :none
         config.action_controller.default_protect_from_forgery = false
       RUBY
 
